@@ -1,13 +1,11 @@
-"""
-Prioritized Experience Replay DQN
-"""
+"""Prioritized Experience Replay DQN."""
 import argparse
 from collections import OrderedDict
 from typing import Tuple
 
 import numpy as np
-import pytorch_lightning as pl
-import torch
+from pytorch_lightning import Trainer
+from torch import Tensor
 from torch.utils.data import DataLoader
 
 from pl_bolts.datamodules import ExperienceSourceDataset
@@ -17,8 +15,7 @@ from pl_bolts.models.rl.dqn_model import DQN
 
 
 class PERDQN(DQN):
-    """
-    PyTorch Lightning implementation of `DQN With Prioritized Experience Replay <https://arxiv.org/abs/1511.05952>`_
+    """PyTorch Lightning implementation of `DQN With Prioritized Experience Replay`_.
 
     Paper authors: Tom Schaul, John Quan, Ioannis Antonoglou, David Silver
 
@@ -43,11 +40,13 @@ class PERDQN(DQN):
 
     .. note:: Currently only supports CPU and single GPU training with `distributed_backend=dp`
 
-        """
+    .. _`DQN With Prioritized Experience Replay`: https://arxiv.org/abs/1511.05952
+    """
 
-    def train_batch(self, ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]:
-        """
-        Contains the logic for generating a new batch of data to be passed to the DataLoader
+    def train_batch(
+        self,
+    ) -> Tuple[Tensor, Tensor, Tensor, Tensor, Tensor]:
+        """Contains the logic for generating a new batch of data to be passed to the DataLoader.
 
         Returns:
             yields a Experience tuple containing the state, action, reward, done and next_state.
@@ -80,7 +79,7 @@ class PERDQN(DQN):
                 self.done_episodes += 1
                 self.total_rewards.append(episode_reward)
                 self.total_episode_steps.append(episode_steps)
-                self.avg_rewards = float(np.mean(self.total_rewards[-self.avg_reward_len:]))
+                self.avg_rewards = float(np.mean(self.total_rewards[-self.avg_reward_len :]))
                 self.state = self.env.reset()
                 episode_steps = 0
                 episode_reward = 0
@@ -90,18 +89,13 @@ class PERDQN(DQN):
             states, actions, rewards, dones, new_states = samples
 
             for idx, _ in enumerate(dones):
-                yield (
-                    states[idx],
-                    actions[idx],
-                    rewards[idx],
-                    dones[idx],
-                    new_states[idx],
-                ), indices[idx], weights[idx]
+                yield (states[idx], actions[idx], rewards[idx], dones[idx], new_states[idx],), indices[
+                    idx
+                ], weights[idx]
 
     def training_step(self, batch, _) -> OrderedDict:
-        """
-        Carries out a single step through the environment to update the replay buffer.
-        Then calculates loss based on the minibatch recieved
+        """Carries out a single step through the environment to update the replay buffer. Then calculates loss
+        based on the minibatch recieved.
 
         Args:
             batch: current mini batch of replay data
@@ -116,7 +110,7 @@ class PERDQN(DQN):
         # calculates training loss
         loss, batch_weights = per_dqn_loss(samples, weights, self.net, self.target_net, self.gamma)
 
-        if self.trainer.use_dp or self.trainer.use_ddp2:
+        if self._use_dp_or_ddp2(self.trainer):
             loss = loss.unsqueeze(0)
 
         # update priorities in buffer
@@ -126,20 +120,24 @@ class PERDQN(DQN):
         if self.global_step % self.sync_rate == 0:
             self.target_net.load_state_dict(self.net.state_dict())
 
-        self.log_dict({
-            "total_reward": self.total_rewards[-1],
-            "avg_reward": self.avg_rewards,
-            "train_loss": loss,
-            # "episodes": self.total_episode_steps,
-        })
+        self.log_dict(
+            {
+                "total_reward": self.total_rewards[-1],
+                "avg_reward": self.avg_rewards,
+                "train_loss": loss,
+                # "episodes": self.total_episode_steps,
+            }
+        )
 
-        return OrderedDict({
-            "loss": loss,
-            "avg_reward": self.avg_rewards,
-        })
+        return OrderedDict(
+            {
+                "loss": loss,
+                "avg_reward": self.avg_rewards,
+            }
+        )
 
     def _dataloader(self) -> DataLoader:
-        """Initialize the Replay Buffer dataset used for retrieving experiences"""
+        """Initialize the Replay Buffer dataset used for retrieving experiences."""
         self.buffer = PERBuffer(self.replay_size)
         self.populate(self.warm_start_size)
 
@@ -151,7 +149,7 @@ def cli_main():
     parser = argparse.ArgumentParser(add_help=False)
 
     # trainer args
-    parser = pl.Trainer.add_argparse_args(parser)
+    parser = Trainer.add_argparse_args(parser)
 
     # model args
     parser = PERDQN.add_model_specific_args(parser)
@@ -159,7 +157,7 @@ def cli_main():
 
     model = PERDQN(**args.__dict__)
 
-    trainer = pl.Trainer.from_argparse_args(args)
+    trainer = Trainer.from_argparse_args(args)
     trainer.fit(model)
 
 
