@@ -1,5 +1,4 @@
-"""
-Adapted from: https://github.com/facebookresearch/moco
+"""Adapted from: https://github.com/facebookresearch/moco.
 
 Original work is: Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 This implementation is: Copyright (c) PyTorch Lightning, Inc. and its affiliates. All Rights Reserved
@@ -12,8 +11,8 @@ You may obtain a copy of the License from the LICENSE file present in this folde
 from argparse import ArgumentParser
 from typing import Union
 
-import pytorch_lightning as pl
 import torch
+from pytorch_lightning import LightningModule, Trainer
 from torch import nn
 from torch.nn import functional as F
 
@@ -26,18 +25,17 @@ from pl_bolts.models.self_supervised.moco.transforms import (
     Moco2TrainImagenetTransforms,
     Moco2TrainSTL10Transforms,
 )
-from pl_bolts.utils import _TORCHVISION_AVAILABLE
+from pl_bolts.utils import _PL_GREATER_EQUAL_1_4, _TORCHVISION_AVAILABLE
 from pl_bolts.utils.warnings import warn_missing_pkg
 
 if _TORCHVISION_AVAILABLE:
     import torchvision
 else:  # pragma: no cover
-    warn_missing_pkg('torchvision')
+    warn_missing_pkg("torchvision")
 
 
-class Moco_v2(pl.LightningModule):
-    """
-    PyTorch Lightning implementation of `Moco <https://arxiv.org/abs/2003.04297>`_
+class Moco_v2(LightningModule):
+    """PyTorch Lightning implementation of `Moco <https://arxiv.org/abs/2003.04297>`_
 
     Paper authors: Xinlei Chen, Haoqi Fan, Ross Girshick, Kaiming He.
 
@@ -67,7 +65,7 @@ class Moco_v2(pl.LightningModule):
 
     def __init__(
         self,
-        base_encoder: Union[str, torch.nn.Module] = 'resnet18',
+        base_encoder: Union[str, torch.nn.Module] = "resnet18",
         emb_dim: int = 128,
         num_negatives: int = 65536,
         encoder_momentum: float = 0.999,
@@ -75,7 +73,7 @@ class Moco_v2(pl.LightningModule):
         learning_rate: float = 0.03,
         momentum: float = 0.9,
         weight_decay: float = 1e-4,
-        data_dir: str = './',
+        data_dir: str = "./",
         batch_size: int = 256,
         use_mlp: bool = False,
         num_workers: int = 8,
@@ -121,10 +119,14 @@ class Moco_v2(pl.LightningModule):
 
         self.register_buffer("queue_ptr", torch.zeros(1, dtype=torch.long))
 
+        # create the validation queue
+        self.register_buffer("val_queue", torch.randn(emb_dim, num_negatives))
+        self.val_queue = nn.functional.normalize(self.val_queue, dim=0)
+
+        self.register_buffer("val_queue_ptr", torch.zeros(1, dtype=torch.long))
+
     def init_encoders(self, base_encoder):
-        """
-        Override to add your own encoders
-        """
+        """Override to add your own encoders."""
 
         template_model = getattr(torchvision.models, base_encoder)
         encoder_q = template_model(num_classes=self.hparams.emb_dim)
@@ -134,34 +136,32 @@ class Moco_v2(pl.LightningModule):
 
     @torch.no_grad()
     def _momentum_update_key_encoder(self):
-        """
-        Momentum update of the key encoder
-        """
+        """Momentum update of the key encoder."""
         for param_q, param_k in zip(self.encoder_q.parameters(), self.encoder_k.parameters()):
             em = self.hparams.encoder_momentum
-            param_k.data = param_k.data * em + param_q.data * (1. - em)
+            param_k.data = param_k.data * em + param_q.data * (1.0 - em)
 
     @torch.no_grad()
-    def _dequeue_and_enqueue(self, keys):
+    def _dequeue_and_enqueue(self, keys, queue_ptr, queue):
         # gather keys before updating queue
-        if self.trainer.use_ddp or self.trainer.use_ddp2:
+        if self._use_ddp_or_ddp2(self.trainer):
             keys = concat_all_gather(keys)
 
         batch_size = keys.shape[0]
 
-        ptr = int(self.queue_ptr)
+        ptr = int(queue_ptr)
         assert self.hparams.num_negatives % batch_size == 0  # for simplicity
 
         # replace the keys at ptr (dequeue and enqueue)
-        self.queue[:, ptr:ptr + batch_size] = keys.T
+        queue[:, ptr : ptr + batch_size] = keys.T
         ptr = (ptr + batch_size) % self.hparams.num_negatives  # move pointer
 
-        self.queue_ptr[0] = ptr
+        queue_ptr[0] = ptr
 
     @torch.no_grad()
     def _batch_shuffle_ddp(self, x):  # pragma: no cover
-        """
-        Batch shuffle, for making use of BatchNorm.
+        """Batch shuffle, for making use of BatchNorm.
+
         *** Only support DistributedDataParallel (DDP) model. ***
         """
         # gather from all gpus
@@ -188,8 +188,8 @@ class Moco_v2(pl.LightningModule):
 
     @torch.no_grad()
     def _batch_unshuffle_ddp(self, x, idx_unshuffle):  # pragma: no cover
-        """
-        Undo batch shuffle.
+        """Undo batch shuffle.
+
         *** Only support DistributedDataParallel (DDP) model. ***
         """
         # gather from all gpus
@@ -205,11 +205,12 @@ class Moco_v2(pl.LightningModule):
 
         return x_gather[idx_this]
 
-    def forward(self, img_q, img_k):
+    def forward(self, img_q, img_k, queue):
         """
         Input:
             im_q: a batch of query images
             im_k: a batch of key images
+            queue: a queue from which to pick negative samples
         Output:
             logits, targets
         """
@@ -220,25 +221,24 @@ class Moco_v2(pl.LightningModule):
 
         # compute key features
         with torch.no_grad():  # no gradient to keys
-            self._momentum_update_key_encoder()  # update the key encoder
 
             # shuffle for making use of BN
-            if self.trainer.use_ddp or self.trainer.use_ddp2:
+            if self._use_ddp_or_ddp2(self.trainer):
                 img_k, idx_unshuffle = self._batch_shuffle_ddp(img_k)
 
             k = self.encoder_k(img_k)  # keys: NxC
             k = nn.functional.normalize(k, dim=1)
 
             # undo shuffle
-            if self.trainer.use_ddp or self.trainer.use_ddp2:
+            if self._use_ddp_or_ddp2(self.trainer):
                 k = self._batch_unshuffle_ddp(k, idx_unshuffle)
 
         # compute logits
         # Einstein sum is more intuitive
         # positive logits: Nx1
-        l_pos = torch.einsum('nc,nc->n', [q, k]).unsqueeze(-1)
+        l_pos = torch.einsum("nc,nc->n", [q, k]).unsqueeze(-1)
         # negative logits: NxK
-        l_neg = torch.einsum('nc,ck->nk', [q, self.queue.clone().detach()])
+        l_neg = torch.einsum("nc,ck->nk", [q, queue.clone().detach()])
 
         # logits: Nx(1+K)
         logits = torch.cat([l_pos, l_neg], dim=1)
@@ -250,52 +250,54 @@ class Moco_v2(pl.LightningModule):
         labels = torch.zeros(logits.shape[0], dtype=torch.long)
         labels = labels.type_as(logits)
 
-        # dequeue and enqueue
-        self._dequeue_and_enqueue(k)
-
-        return logits, labels
+        return logits, labels, k
 
     def training_step(self, batch, batch_idx):
         # in STL10 we pass in both lab+unl for online ft
-        if self.trainer.datamodule.name == 'stl10':
+        if self.trainer.datamodule.name == "stl10":
             # labeled_batch = batch[1]
             unlabeled_batch = batch[0]
             batch = unlabeled_batch
 
         (img_1, img_2), _ = batch
 
-        output, target = self(img_q=img_1, img_k=img_2)
+        self._momentum_update_key_encoder()  # update the key encoder
+        output, target, keys = self(img_q=img_1, img_k=img_2, queue=self.queue)
+        self._dequeue_and_enqueue(keys, queue=self.queue, queue_ptr=self.queue_ptr)  # dequeue and enqueue
+
         loss = F.cross_entropy(output.float(), target.long())
 
         acc1, acc5 = precision_at_k(output, target, top_k=(1, 5))
 
-        log = {'train_loss': loss, 'train_acc1': acc1, 'train_acc5': acc5}
+        log = {"train_loss": loss, "train_acc1": acc1, "train_acc5": acc5}
         self.log_dict(log)
         return loss
 
     def validation_step(self, batch, batch_idx):
         # in STL10 we pass in both lab+unl for online ft
-        if self.trainer.datamodule.name == 'stl10':
+        if self.trainer.datamodule.name == "stl10":
             # labeled_batch = batch[1]
             unlabeled_batch = batch[0]
             batch = unlabeled_batch
 
         (img_1, img_2), labels = batch
 
-        output, target = self(img_q=img_1, img_k=img_2)
+        output, target, keys = self(img_q=img_1, img_k=img_2, queue=self.val_queue)
+        self._dequeue_and_enqueue(keys, queue=self.val_queue, queue_ptr=self.val_queue_ptr)  # dequeue and enqueue
+
         loss = F.cross_entropy(output, target.long())
 
         acc1, acc5 = precision_at_k(output, target, top_k=(1, 5))
 
-        results = {'val_loss': loss, 'val_acc1': acc1, 'val_acc5': acc5}
+        results = {"val_loss": loss, "val_acc1": acc1, "val_acc5": acc5}
         return results
 
     def validation_epoch_end(self, outputs):
-        val_loss = mean(outputs, 'val_loss')
-        val_acc1 = mean(outputs, 'val_acc1')
-        val_acc5 = mean(outputs, 'val_acc5')
+        val_loss = mean(outputs, "val_loss")
+        val_acc1 = mean(outputs, "val_acc1")
+        val_acc5 = mean(outputs, "val_acc5")
 
-        log = {'val_loss': val_loss, 'val_acc1': val_acc1, 'val_acc5': val_acc5}
+        log = {"val_loss": val_loss, "val_acc1": val_acc1, "val_acc5": val_acc5}
         self.log_dict(log)
 
     def configure_optimizers(self):
@@ -303,36 +305,43 @@ class Moco_v2(pl.LightningModule):
             self.parameters(),
             self.hparams.learning_rate,
             momentum=self.hparams.momentum,
-            weight_decay=self.hparams.weight_decay
+            weight_decay=self.hparams.weight_decay,
         )
         return optimizer
 
     @staticmethod
     def add_model_specific_args(parent_parser):
         parser = ArgumentParser(parents=[parent_parser], add_help=False)
-        parser.add_argument('--base_encoder', type=str, default='resnet18')
-        parser.add_argument('--emb_dim', type=int, default=128)
-        parser.add_argument('--num_workers', type=int, default=8)
-        parser.add_argument('--num_negatives', type=int, default=65536)
-        parser.add_argument('--encoder_momentum', type=float, default=0.999)
-        parser.add_argument('--softmax_temperature', type=float, default=0.07)
-        parser.add_argument('--learning_rate', type=float, default=0.03)
-        parser.add_argument('--momentum', type=float, default=0.9)
-        parser.add_argument('--weight_decay', type=float, default=1e-4)
-        parser.add_argument('--data_dir', type=str, default='./')
-        parser.add_argument('--dataset', type=str, default='cifar10', choices=['cifar10', 'imagenet2012', 'stl10'])
-        parser.add_argument('--batch_size', type=int, default=256)
-        parser.add_argument('--use_mlp', action='store_true')
-        parser.add_argument('--meta_dir', default='.', type=str, help='path to meta.bin for imagenet')
+        parser.add_argument("--base_encoder", type=str, default="resnet18")
+        parser.add_argument("--emb_dim", type=int, default=128)
+        parser.add_argument("--num_workers", type=int, default=8)
+        parser.add_argument("--num_negatives", type=int, default=65536)
+        parser.add_argument("--encoder_momentum", type=float, default=0.999)
+        parser.add_argument("--softmax_temperature", type=float, default=0.07)
+        parser.add_argument("--learning_rate", type=float, default=0.03)
+        parser.add_argument("--momentum", type=float, default=0.9)
+        parser.add_argument("--weight_decay", type=float, default=1e-4)
+        parser.add_argument("--data_dir", type=str, default="./")
+        parser.add_argument("--dataset", type=str, default="cifar10", choices=["cifar10", "imagenet2012", "stl10"])
+        parser.add_argument("--batch_size", type=int, default=256)
+        parser.add_argument("--use_mlp", action="store_true")
+        parser.add_argument("--meta_dir", default=".", type=str, help="path to meta.bin for imagenet")
 
         return parser
 
+    @staticmethod
+    def _use_ddp_or_ddp2(trainer: Trainer) -> bool:
+        # for backwards compatibility
+        if _PL_GREATER_EQUAL_1_4:
+            return trainer.accelerator_connector.use_ddp or trainer.accelerator_connector.use_ddp2
+        return trainer.use_ddp or trainer.use_ddp2
+
 
 # utils
 @torch.no_grad()
 def concat_all_gather(tensor):
-    """
-    Performs all_gather operation on the provided tensors.
+    """Performs all_gather operation on the provided tensors.
+
     *** Warning ***: torch.distributed.all_gather has no gradient.
     """
     tensors_gather = [torch.ones_like(tensor) for _ in range(torch.distributed.get_world_size())]
@@ -348,25 +357,25 @@ def cli_main():
     parser = ArgumentParser()
 
     # trainer args
-    parser = pl.Trainer.add_argparse_args(parser)
+    parser = Trainer.add_argparse_args(parser)
 
     # model args
     parser = Moco_v2.add_model_specific_args(parser)
     args = parser.parse_args()
 
-    if args.dataset == 'cifar10':
+    if args.dataset == "cifar10":
         datamodule = CIFAR10DataModule.from_argparse_args(args)
         datamodule.train_transforms = Moco2TrainCIFAR10Transforms()
         datamodule.val_transforms = Moco2EvalCIFAR10Transforms()
 
-    elif args.dataset == 'stl10':
+    elif args.dataset == "stl10":
         datamodule = STL10DataModule.from_argparse_args(args)
         datamodule.train_dataloader = datamodule.train_dataloader_mixed
         datamodule.val_dataloader = datamodule.val_dataloader_mixed
         datamodule.train_transforms = Moco2TrainSTL10Transforms()
         datamodule.val_transforms = Moco2EvalSTL10Transforms()
 
-    elif args.dataset == 'imagenet2012':
+    elif args.dataset == "imagenet2012":
         datamodule = SSLImagenetDataModule.from_argparse_args(args)
         datamodule.train_transforms = Moco2TrainImagenetTransforms()
         datamodule.val_transforms = Moco2EvalImagenetTransforms()
@@ -377,9 +386,9 @@ def cli_main():
 
     model = Moco_v2(**args.__dict__)
 
-    trainer = pl.Trainer.from_argparse_args(args)
+    trainer = Trainer.from_argparse_args(args)
     trainer.fit(model, datamodule=datamodule)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     cli_main()
