@@ -1,13 +1,11 @@
 from typing import Optional, Tuple
 
-import torch
-from torch import nn
+from torch import Tensor, nn
 
 from pl_bolts.utils.self_supervised import torchvision_ssl_encoder
 
 
 class MLP(nn.Module):
-
     def __init__(self, input_dim: int = 2048, hidden_size: int = 4096, output_dim: int = 256) -> None:
         super().__init__()
         self.output_dim = output_dim
@@ -19,13 +17,12 @@ class MLP(nn.Module):
             nn.Linear(hidden_size, output_dim, bias=True),
         )
 
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
+    def forward(self, x: Tensor) -> Tensor:
         x = self.model(x)
         return x
 
 
 class SiameseArm(nn.Module):
-
     def __init__(
         self,
         encoder: Optional[nn.Module] = None,
@@ -36,7 +33,7 @@ class SiameseArm(nn.Module):
         super().__init__()
 
         if encoder is None:
-            encoder = torchvision_ssl_encoder('resnet50')
+            encoder = torchvision_ssl_encoder("resnet50")
         # Encoder
         self.encoder = encoder
         # Projector
@@ -44,7 +41,7 @@ class SiameseArm(nn.Module):
         # Predictor
         self.predictor = MLP(output_dim, hidden_size, output_dim)
 
-    def forward(self, x: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
+    def forward(self, x: Tensor) -> Tuple[Tensor, Tensor, Tensor]:
         y = self.encoder(x)[0]
         z = self.projector(y)
         h = self.predictor(z)
