@@ -1,22 +1,17 @@
-"""
-Adapted from official swav implementation: https://github.com/facebookresearch/swav
-"""
-import math
+"""Adapted from official swav implementation: https://github.com/facebookresearch/swav."""
 import os
 from argparse import ArgumentParser
-from typing import Callable, Optional
 
 import numpy as np
-import pytorch_lightning as pl
 import torch
-from pytorch_lightning.callbacks import ModelCheckpoint
-from pytorch_lightning.core.optimizer import LightningOptimizer
+from pytorch_lightning import LightningModule, Trainer
+from pytorch_lightning.callbacks import LearningRateMonitor, ModelCheckpoint
 from torch import distributed as dist
 from torch import nn
-from torch.optim.optimizer import Optimizer
 
 from pl_bolts.models.self_supervised.swav.swav_resnet import resnet18, resnet50
-from pl_bolts.optimizers.lars_scheduling import LARSWrapper
+from pl_bolts.optimizers.lars import LARS
+from pl_bolts.optimizers.lr_scheduler import linear_warmup_decay
 from pl_bolts.transforms.dataset_normalizations import (
     cifar10_normalization,
     imagenet_normalization,
@@ -24,8 +19,7 @@ from pl_bolts.transforms.dataset_normalizations import (
 )
 
 
-class SwAV(pl.LightningModule):
-
+class SwAV(LightningModule):
     def __init__(
         self,
         gpus: int,
@@ -33,7 +27,7 @@ class SwAV(pl.LightningModule):
         batch_size: int,
         dataset: str,
         num_nodes: int = 1,
-        arch: str = 'resnet50',
+        arch: str = "resnet50",
         hidden_mlp: int = 2048,
         feat_dim: int = 128,
         warmup_epochs: int = 10,
@@ -49,12 +43,11 @@ class SwAV(pl.LightningModule):
         nmb_crops: list = [2, 6],
         first_conv: bool = True,
         maxpool1: bool = True,
-        optimizer: str = 'adam',
-        lars_wrapper: bool = True,
+        optimizer: str = "adam",
         exclude_bn_bias: bool = False,
-        start_lr: float = 0.,
+        start_lr: float = 0.0,
         learning_rate: float = 1e-3,
-        final_lr: float = 0.,
+        final_lr: float = 0.0,
         weight_decay: float = 1e-6,
         epsilon: float = 0.05,
         **kwargs
@@ -90,7 +83,6 @@ class SwAV(pl.LightningModule):
             maxpool1: keep first maxpool layer same as the original resnet architecture,
                 if set to false, first maxpool is turned off (cifar10, maybe stl10)
             optimizer: optimizer to use
-            lars_wrapper: use LARS wrapper over the optimizer
             exclude_bn_bias: exclude batchnorm and bias layers from weight decay in optimizers
             start_lr: starting lr for linear warmup
             learning_rate: learning rate
@@ -124,7 +116,6 @@ class SwAV(pl.LightningModule):
         self.maxpool1 = maxpool1
 
         self.optim = optimizer
-        self.lars_wrapper = lars_wrapper
         self.exclude_bn_bias = exclude_bn_bias
         self.weight_decay = weight_decay
         self.epsilon = epsilon
@@ -136,7 +127,7 @@ class SwAV(pl.LightningModule):
         self.warmup_epochs = warmup_epochs
         self.max_epochs = max_epochs
 
-        if self.gpus or self.num_nodes > 1:
+        if self.gpus * self.num_nodes > 1:
             self.get_assignments = self.distributed_sinkhorn
         else:
             self.get_assignments = self.sinkhorn
@@ -144,24 +135,9 @@ class SwAV(pl.LightningModule):
         self.model = self.init_model()
 
         # compute iters per epoch
-        nb_gpus = len(self.gpus) if isinstance(gpus, (list, tuple)) else self.gpus
-        assert isinstance(nb_gpus, int)
-        global_batch_size = self.num_nodes * nb_gpus * self.batch_size if nb_gpus > 0 else self.batch_size
+        global_batch_size = self.num_nodes * self.gpus * self.batch_size if self.gpus > 0 else self.batch_size
         self.train_iters_per_epoch = self.num_samples // global_batch_size
 
-        # define LR schedule
-        warmup_lr_schedule = np.linspace(
-            self.start_lr, self.learning_rate, self.train_iters_per_epoch * self.warmup_epochs
-        )
-        iters = np.arange(self.train_iters_per_epoch * (self.max_epochs - self.warmup_epochs))
-        cosine_lr_schedule = np.array([
-            self.final_lr + 0.5 * (self.learning_rate - self.final_lr) *
-            (1 + math.cos(math.pi * t / (self.train_iters_per_epoch * (self.max_epochs - self.warmup_epochs))))
-            for t in iters
-        ])
-
-        self.lr_schedule = np.concatenate((warmup_lr_schedule, cosine_lr_schedule))
-
         self.queue = None
         self.softmax = nn.Softmax(dim=1)
 
@@ -177,9 +153,9 @@ class SwAV(pl.LightningModule):
                 self.queue = torch.load(self.queue_path)["queue"]
 
     def init_model(self):
-        if self.arch == 'resnet18':
+        if self.arch == "resnet18":
             backbone = resnet18
-        elif self.arch == 'resnet50':
+        elif self.arch == "resnet50":
             backbone = resnet50
 
         return backbone(
@@ -188,7 +164,7 @@ class SwAV(pl.LightningModule):
             output_dim=self.feat_dim,
             nmb_prototypes=self.nmb_prototypes,
             first_conv=self.first_conv,
-            maxpool1=self.maxpool1
+            maxpool1=self.maxpool1,
         )
 
     def forward(self, x):
@@ -204,8 +180,8 @@ class SwAV(pl.LightningModule):
                     self.feat_dim,
                 )
 
-                if self.gpus > 0:
-                    self.queue = self.queue.cuda()
+            if self.queue is not None:
+                self.queue = self.queue.to(self.device)
 
         self.use_the_queue = False
 
@@ -220,7 +196,7 @@ class SwAV(pl.LightningModule):
                     p.grad = None
 
     def shared_step(self, batch):
-        if self.dataset == 'stl10':
+        if self.dataset == "stl10":
             unlabeled_batch = batch[0]
             batch = unlabeled_batch
 
@@ -242,7 +218,7 @@ class SwAV(pl.LightningModule):
         loss = 0
         for i, crop_id in enumerate(self.crops_for_assign):
             with torch.no_grad():
-                out = output[bs * crop_id:bs * (crop_id + 1)]
+                out = output[bs * crop_id : bs * (crop_id + 1)]
 
                 # 4. time to use the queue
                 if self.queue is not None:
@@ -251,7 +227,7 @@ class SwAV(pl.LightningModule):
                         out = torch.cat((torch.mm(self.queue[i], self.model.prototypes.weight.t()), out))
                     # fill the queue
                     self.queue[i, bs:] = self.queue[i, :-bs].clone()
-                    self.queue[i, :bs] = embedding[crop_id * bs:(crop_id + 1) * bs]
+                    self.queue[i, :bs] = embedding[crop_id * bs : (crop_id + 1) * bs]
 
                 # 5. get assignments
                 q = torch.exp(out / self.epsilon).t()
@@ -260,7 +236,7 @@ class SwAV(pl.LightningModule):
             # cluster assignment prediction
             subloss = 0
             for v in np.delete(np.arange(np.sum(self.nmb_crops)), crop_id):
-                p = self.softmax(output[bs * v:bs * (v + 1)] / self.temperature)
+                p = self.softmax(output[bs * v : bs * (v + 1)] / self.temperature)
                 subloss -= torch.mean(torch.sum(q * torch.log(p), dim=1))
             loss += subloss / (np.sum(self.nmb_crops) - 1)
         loss /= len(self.crops_for_assign)
@@ -270,19 +246,16 @@ class SwAV(pl.LightningModule):
     def training_step(self, batch, batch_idx):
         loss = self.shared_step(batch)
 
-        # log LR (LearningRateLogger callback doesn't work with LARSWrapper)
-        self.log('learning_rate', self.lr_schedule[self.trainer.global_step], on_step=True, on_epoch=False)
-
-        self.log('train_loss', loss, on_step=True, on_epoch=False)
+        self.log("train_loss", loss, on_step=True, on_epoch=False)
         return loss
 
     def validation_step(self, batch, batch_idx):
         loss = self.shared_step(batch)
 
-        self.log('val_loss', loss, on_step=False, on_epoch=True)
+        self.log("val_loss", loss, on_step=False, on_epoch=True)
         return loss
 
-    def exclude_from_wt_decay(self, named_params, weight_decay, skip_list=['bias', 'bn']):
+    def exclude_from_wt_decay(self, named_params, weight_decay, skip_list=["bias", "bn"]):
         params = []
         excluded_params = []
 
@@ -294,7 +267,7 @@ class SwAV(pl.LightningModule):
             else:
                 params.append(param)
 
-        return [{'params': params, 'weight_decay': weight_decay}, {'params': excluded_params, 'weight_decay': 0.}]
+        return [{"params": params, "weight_decay": weight_decay}, {"params": excluded_params, "weight_decay": 0.0}]
 
     def configure_optimizers(self):
         if self.exclude_bn_bias:
@@ -302,41 +275,30 @@ class SwAV(pl.LightningModule):
         else:
             params = self.parameters()
 
-        if self.optim == 'sgd':
-            optimizer = torch.optim.SGD(params, lr=self.learning_rate, momentum=0.9, weight_decay=self.weight_decay)
-        elif self.optim == 'adam':
+        if self.optim == "lars":
+            optimizer = LARS(
+                params,
+                lr=self.learning_rate,
+                momentum=0.9,
+                weight_decay=self.weight_decay,
+                trust_coefficient=0.001,
+            )
+        elif self.optim == "adam":
             optimizer = torch.optim.Adam(params, lr=self.learning_rate, weight_decay=self.weight_decay)
 
-        if self.lars_wrapper:
-            optimizer = LARSWrapper(
+        warmup_steps = self.train_iters_per_epoch * self.warmup_epochs
+        total_steps = self.train_iters_per_epoch * self.max_epochs
+
+        scheduler = {
+            "scheduler": torch.optim.lr_scheduler.LambdaLR(
                 optimizer,
-                eta=0.001,  # trust coefficient
-                clip=False
-            )
+                linear_warmup_decay(warmup_steps, total_steps, cosine=True),
+            ),
+            "interval": "step",
+            "frequency": 1,
+        }
 
-        return optimizer
-
-    def optimizer_step(
-        self,
-        epoch: int = None,
-        batch_idx: int = None,
-        optimizer: Optimizer = None,
-        optimizer_idx: int = None,
-        optimizer_closure: Optional[Callable] = None,
-        on_tpu: bool = None,
-        using_native_amp: bool = None,
-        using_lbfgs: bool = None,
-    ) -> None:
-        # warm-up + decay schedule placed here since LARSWrapper is not optimizer class
-        # adjust LR of optim contained within LARSWrapper
-        for param_group in optimizer.param_groups:
-            param_group["lr"] = self.lr_schedule[self.trainer.global_step]
-
-        # from lightning
-        if not isinstance(optimizer, LightningOptimizer):
-            # wraps into LightingOptimizer only for running step
-            optimizer = LightningOptimizer.to_lightning_optimizer(optimizer, self.trainer)
-        optimizer.step(closure=optimizer_closure)
+        return [optimizer], [scheduler]
 
     def sinkhorn(self, Q, nmb_iters):
         with torch.no_grad():
@@ -395,12 +357,12 @@ class SwAV(pl.LightningModule):
         # model params
         parser.add_argument("--arch", default="resnet50", type=str, help="convnet architecture")
         # specify flags to store false
-        parser.add_argument("--first_conv", action='store_false')
-        parser.add_argument("--maxpool1", action='store_false')
+        parser.add_argument("--first_conv", action="store_false")
+        parser.add_argument("--maxpool1", action="store_false")
         parser.add_argument("--hidden_mlp", default=2048, type=int, help="hidden layer dimension in projection head")
         parser.add_argument("--feat_dim", default=128, type=int, help="feature dimension")
-        parser.add_argument("--online_ft", action='store_true')
-        parser.add_argument("--fp32", action='store_true')
+        parser.add_argument("--online_ft", action="store_true")
+        parser.add_argument("--fp32", action="store_true")
 
         # transform params
         parser.add_argument("--gaussian_blur", action="store_true", help="add gaussian blur")
@@ -420,21 +382,25 @@ class SwAV(pl.LightningModule):
             type=float,
             default=[0.33, 0.10],
             nargs="+",
-            help="argument in RandomResizedCrop (example: [0.14, 0.05])"
+            help="argument in RandomResizedCrop (example: [0.14, 0.05])",
         )
         parser.add_argument(
             "--max_scale_crops",
             type=float,
             default=[1, 0.33],
             nargs="+",
-            help="argument in RandomResizedCrop (example: [1., 0.14])"
+            help="argument in RandomResizedCrop (example: [1., 0.14])",
         )
 
         # training params
+        parser.add_argument("--fast_dev_run", default=1, type=int)
+        parser.add_argument("--num_nodes", default=1, type=int, help="number of nodes for training")
+        parser.add_argument("--gpus", default=1, type=int, help="number of gpus to train on")
         parser.add_argument("--num_workers", default=8, type=int, help="num of workers per GPU")
-        parser.add_argument("--optimizer", default="adam", type=str, help="choose between adam/sgd")
-        parser.add_argument("--lars_wrapper", action='store_true', help="apple lars wrapper over optimizer used")
-        parser.add_argument('--exclude_bn_bias', action='store_true', help="exclude bn/bias from weight decay")
+        parser.add_argument("--optimizer", default="adam", type=str, help="choose between adam/lars")
+        parser.add_argument("--exclude_bn_bias", action="store_true", help="exclude bn/bias from weight decay")
+        parser.add_argument("--max_epochs", default=100, type=int, help="number of total epochs to run")
+        parser.add_argument("--max_steps", default=-1, type=int, help="max steps")
         parser.add_argument("--warmup_epochs", default=10, type=int, help="number of warmup epochs")
         parser.add_argument("--batch_size", default=128, type=int, help="batch size per gpu")
 
@@ -449,7 +415,7 @@ class SwAV(pl.LightningModule):
             type=int,
             nargs="+",
             default=[0, 1],
-            help="list of crops id used for computing assignments"
+            help="list of crops id used for computing assignments",
         )
         parser.add_argument("--temperature", default=0.1, type=float, help="temperature parameter in training loss")
         parser.add_argument(
@@ -463,7 +429,7 @@ class SwAV(pl.LightningModule):
             "--queue_length",
             type=int,
             default=0,
-            help="length of the queue (0 for no queue); must be divisible by total batch size"
+            help="length of the queue (0 for no queue); must be divisible by total batch size",
         )
         parser.add_argument(
             "--epoch_queue_starts", type=int, default=15, help="from this epoch, we start using a queue"
@@ -472,7 +438,7 @@ class SwAV(pl.LightningModule):
             "--freeze_prototypes_epochs",
             default=1,
             type=int,
-            help="freeze the prototypes during this many epochs from the start"
+            help="freeze the prototypes during this many epochs from the start",
         )
 
         return parser
@@ -487,10 +453,9 @@ def cli_main():
 
     # model args
     parser = SwAV.add_model_specific_args(parser)
-    parser = pl.Trainer.add_argparse_args(parser)
     args = parser.parse_args()
 
-    if args.dataset == 'stl10':
+    if args.dataset == "stl10":
         dm = STL10DataModule(data_dir=args.data_dir, batch_size=args.batch_size, num_workers=args.num_workers)
 
         dm.train_dataloader = dm.train_dataloader_mixed
@@ -500,7 +465,7 @@ def cli_main():
         args.maxpool1 = False
 
         normalization = stl10_normalization()
-    elif args.dataset == 'cifar10':
+    elif args.dataset == "cifar10":
         args.batch_size = 2
         args.num_workers = 0
 
@@ -517,7 +482,7 @@ def cli_main():
         args.size_crops = [32, 16]
         args.nmb_crops = [2, 1]
         args.gaussian_blur = False
-    elif args.dataset == 'imagenet':
+    elif args.dataset == "imagenet":
         args.maxpool1 = True
         args.first_conv = True
         normalization = imagenet_normalization()
@@ -525,17 +490,16 @@ def cli_main():
         args.size_crops = [224, 96]
         args.nmb_crops = [2, 6]
         args.min_scale_crops = [0.14, 0.05]
-        args.max_scale_crops = [1., 0.14]
+        args.max_scale_crops = [1.0, 0.14]
         args.gaussian_blur = True
-        args.jitter_strength = 1.
+        args.jitter_strength = 1.0
 
         args.batch_size = 64
         args.num_nodes = 8
         args.gpus = 8  # per-node
         args.max_epochs = 800
 
-        args.optimizer = 'sgd'
-        args.lars_wrapper = True
+        args.optimizer = "lars"
         args.learning_rate = 4.8
         args.final_lr = 0.0048
         args.start_lr = 0.3
@@ -557,7 +521,7 @@ def cli_main():
         min_scale_crops=args.min_scale_crops,
         max_scale_crops=args.max_scale_crops,
         gaussian_blur=args.gaussian_blur,
-        jitter_strength=args.jitter_strength
+        jitter_strength=args.jitter_strength,
     )
 
     dm.val_transforms = SwAVEvalDataTransform(
@@ -567,7 +531,7 @@ def cli_main():
         min_scale_crops=args.min_scale_crops,
         max_scale_crops=args.max_scale_crops,
         gaussian_blur=args.gaussian_blur,
-        jitter_strength=args.jitter_strength
+        jitter_strength=args.jitter_strength,
     )
 
     # swav model init
@@ -577,24 +541,32 @@ def cli_main():
     if args.online_ft:
         # online eval
         online_evaluator = SSLOnlineEvaluator(
-            drop_p=0.,
+            drop_p=0.0,
             hidden_dim=None,
             z_dim=args.hidden_mlp,
             num_classes=dm.num_classes,
             dataset=args.dataset,
         )
 
-    model_checkpoint = ModelCheckpoint(save_last=True, save_top_k=1, monitor='val_loss')
+    lr_monitor = LearningRateMonitor(logging_interval="step")
+    model_checkpoint = ModelCheckpoint(save_last=True, save_top_k=1, monitor="val_loss")
     callbacks = [model_checkpoint, online_evaluator] if args.online_ft else [model_checkpoint]
+    callbacks.append(lr_monitor)
 
-    trainer = pl.Trainer.from_argparse_args(
-        args,
+    trainer = Trainer(
+        max_epochs=args.max_epochs,
+        max_steps=None if args.max_steps == -1 else args.max_steps,
+        gpus=args.gpus,
+        num_nodes=args.num_nodes,
+        distributed_backend="ddp" if args.gpus > 1 else None,
         sync_batchnorm=True if args.gpus > 1 else False,
+        precision=32 if args.fp32 else 16,
         callbacks=callbacks,
+        fast_dev_run=args.fast_dev_run,
     )
 
     trainer.fit(model, datamodule=dm)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     cli_main()
